/* Manual Rotor Control
	Contorlla il driver RotorDRV da tastiera pur lasciando libera la
	gestione da parte dell'OrbitDRV o Instant Track

	Versione 3: Calibrazione offset per l'installazione. Per
		    l'istallazione si procede come segue:
		    (i) si porta il rotore a AZ=0 con AZo=0 a terra
		    (ii) si issa il rotore cercando di farlo puntare a
		    340-350 e si blocca in tale posizione
		    (iii) si cerca una sorgente di direzione nota e vi si
		    puntano le antenne lasciando AZo=0
		    (iv) si pone AZo uguale alla differenza fra AZ e la
		    direzione vera conosciuta
		    (v) per verifica si porta AZ sul valore della direzione
		    vera conosciuta e li ci deve essere il massimo segnale

		    Si potrebbe fare lo stesso per l'EL (cioe' montando le
		    antenne una decina di gradi puntate verso il basso quando
		    EL=0 e ELo=0, e seguire la stessa procedura) ma e' piu'
		    facile montare il palo con il filo a piombo; percio' si
		    lascia ELo=0.
*/

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <dos.h>

#define INTNO 0x63


void strscr(void), help(void), up(void), down(void), ccw(void), cw(void);
void azimset(void), elevset(void), displ(void), indrv(void), outdrv(void);
void toggleflip(void), offsetset(void);
int intchk(void);
void interrupt (*old)(), perdispl();

int status, azim, elev, flip;


void offsetset(void)
{
char buff[602];
int azo, elo;
setvect(0x1c,old);
gettext(3,3,77,6,buff);
window(3,3,77,6);
textbackground(CYAN);
textcolor(BLACK);
clrscr();
gotoxy(24,1);
cprintf("Regolazione Offset");
gotoxy(10,3); cprintf("AZoffset = "); cscanf("%d",&azo);
gotoxy(40,3); cprintf("ELoffset = "); cscanf("%d",&elo);
gotoxy(5,4);
cprintf("Non funziona: va fatto il driver con una funzione per l'offset");
delay(1000);
puttext(3,3,77,6,buff);
window(1,1,80,10);
textbackground(BLACK);
textcolor(WHITE);
setvect(0x1c,perdispl);

}

void interrupt perdispl(){
if(status>0)
	{
	indrv();
	gotoxy(4,6);
	cprintf("    Azim %4d                       Elev %4d            Flip %d", azim, elev, flip);
	}
old();
}


void main(void){
int loop=1;

indrv();

old=getvect(0x1c);
setvect(0x1c,perdispl);

clrscr();
window(1,1,80,10);
strscr();
status=intchk();  // sulla 4 riga vengo stampate le info simili all'INT63.EXE
displ();
while(loop){
	while(!kbhit()) ;
	switch(getch())
		{
		case 'a':
		case 'A': azimset(); break;
		case 'e':
		case 'E': elevset(); break;
		case 'o':
		case 'O': offsetset(); break;
		case 0: switch(getch())
			{
			case 45: loop=0; break;    // Alt X exit
			case 59: help(); break;    // F1 help
			case 72: up(); break;
			case 80: down(); break;
			case 75: ccw(); break;
			case 77: cw(); break;
			case 33: toggleflip(); break;  //Alt F
			}
		}
	}
clrscr();
window(1,1,80,25);
setvect(0x1c, old);
}

void strscr(void){
textcolor(CYAN);
cprintf("Ŀ");
cprintf("  Manual  Rotor  Control  (for Instant Track compatible driver)    by IK5NAX  ");
cprintf("                                                                              ");
cprintf("                                                                              ");
cprintf("                                                                              ");
cprintf("                                                                              ");
cprintf("                                                                              ");
cprintf(" Alt X Exit   F1 Help  A set azim  E set elev  Alt F Flip  O Offset  [Cursor] ");
//cprintf(" Alt X  Exit    F1  Help   A  set azim    E  set elev  Alt F  Flip   [Cursor] ");
cprintf("");
}

int intchk(void){
int segp, offp, i, seg, off;
char buff[50];

// Ricerca indirizzo routine interrupt INTNO (0x63)
// vedere l'intervue per l'utilizzo della funzione 35 dell' INT 21
// (la funzione 1202 dell'INT 2F aveva dato problemi sotto Win)

asm{
mov ah,35h
mov al,INTNO
int 21h
}
seg=_ES;
off=_BX;

/*
// calcolo dell'indirizzo
seg=peek(segp,offp+2);
off=peek(segp,offp);
*/

// se l'indirizzo e' 0000:0000 vuol dire che l'INT 63 non e' usato
if(seg==0x0000&&off==0x0000)
	{
	gotoxy(5,4);
	textcolor(LIGHTGRAY+BLINK);
	cprintf("INT%2X not used", INTNO);
	return -1;
	}

// cerca la stringa RotorDRV
for(i=10;i<50;i++)
	{
	buff[i-10]=peek(seg,off+i);
	}

if(strncmp(buff,"RotorDRV",8)==0)
	{
	gotoxy(5,4);
	textcolor(YELLOW);
	if(buff[8]=='+')
		cprintf("Found at INT%2X: %s\n",INTNO, buff);
	else
		cprintf("Found at INT%2X: RotorDRV", INTNO);
	return 1;
	}
else
     {
     gotoxy(5,4);
     textcolor(LIGHTGRAY+BLINK);
     cprintf("Invalid RotorDRV attached to INT%2X",INTNO);
     return 0;
     }
}

void displ(void){
//gotoxy(4,6);
if(status>0)
	{
	textcolor(WHITE);
	outdrv();
	indrv();
	gotoxy(4,6);
	cprintf("    Azim %4d                       Elev %4d            Flip %d", azim, elev, flip);
	}
else
	{
	textcolor(DARKGRAY);
	gotoxy(4,6);
	cprintf("    Azim %4d                       Elev %4d            Flip %d", azim, elev, flip);
	}
}

void up(void){
elev+=2;
if(elev>90) elev=90;
displ();
}

void down(void){
elev-=2;
if(elev<0) elev=0;
displ();
}

void cw(void){
azim+=2;
if(azim>359) azim=359;
displ();
}


void ccw(void){
azim-=2;
if(azim<0) azim=0;
displ();
}

void help(void){
char buff[480];
setvect(0x1c,old);
gettext(3,4,77,6,buff);
window(3,4,77,6);
clrscr();
textcolor(LIGHTRED);
gotoxy(10,2);
cprintf(".....................Help Screeen...................");
getch();
puttext(3,4,77,6,buff);
window(1,1,80,10);
setvect(0x1c,perdispl);
}

void azimset(void){
setvect(0x1c,old);
window(19,6,25,6);
textcolor(WHITE);
textbackground(MAGENTA);
clrscr();
cprintf(" = ");
cscanf("%d",&azim);
if(azim>359) azim=359;
if(azim<0) azim=0;
textbackground(BLACK);
clrscr();
window(1,1,80,10);
displ();
setvect(0x1c,perdispl);
}

void elevset(void){
setvect(0x1c,old);
window(52,6,58,6);
textcolor(WHITE);
textbackground(MAGENTA);
clrscr();
cprintf(" = ");
cscanf("%d",&elev);
if(elev>90) elev=90;
if(elev<0) elev=0;
textbackground(BLACK);
clrscr();
window(1,1,80,10);
displ();
setvect(0x1c,perdispl);
}

void indrv(void){
_AH=0x03;
geninterrupt(INTNO);
azim=_DX;
elev=_CX;

_AH=0x41;
geninterrupt(INTNO);
flip=_AL;
}

void outdrv(void){
_AH=0x04;
_DX=azim;
_CX=elev;
geninterrupt(INTNO);
}

void toggleflip(void){
_AH=0x41;
geninterrupt(INTNO);
flip=_AL;

flip^=1;

_AH=0x40;
_AL=flip;
geninterrupt(INTNO);
displ();
}
